@import url(pygments.css);
body {
  background-color: white; }

#blogheader {
  margin-bottom: 10px;
  padding: 10px;
  font-family: Palatino, "Times New Roman", serif;
  text-decoration: none;
  text-align: center; }
  #blogheader #blogtitle h2 {
    padding-top: 1.5em; }
    #blogheader #blogtitle h2 a {
      color: black;
      text-decoration: none; }
  #blogheader #mainlinks {
    padding: 1em; }
    #blogheader #mainlinks li {
      display: inline-block;
      margin: 0 2em 0 2em; }
    #blogheader #mainlinks a {
      color: black;
      text-decoration: none;
      font-family: Palatino, "Times New Roman", serif; }
      #blogheader #mainlinks a:hover {
        text-shadow: #888888 0px 0px 1px; }

#blogbody {
  margin: 0 auto;
  width: 800px; }
  #blogbody .posting {
    padding: 1em;
    margin-top: 64px;
    margin-bottom: 64px; }
    #blogbody .posting .postheader .subject {
      margin-bottom: -0.7em; }
      #blogbody .posting .postheader .subject a {
        color: black;
        font: bold medium Palatino, "Times New Roman";
        text-decoration: none; }
        #blogbody .posting .postheader .subject a:hover {
          text-shadow: #aaaaaa 0px 0px 2px; }
    #blogbody .posting .postheader .date {
      font: 0.7em "Georgia";
      float: right; }
    #blogbody .posting .postbody {
      text-align: justify;
      padding: 1em;
      font-family: Palatino, "Times New Roman", serif;
      font-size: 11pt; }
      #blogbody .posting .postbody h2 {
        font: 12pt Palatino, "Times New Roman", serif;
        font-weight: bold;
        color: #888888;
        border-bottom: 1px dotted #888888; }
      #blogbody .posting .postbody h3, #blogbody .posting .postbody h4 {
        font: 12pt Palatino, "Times New Roman", serif;
        font-weight: bold;
        color: #888888;
        padding-left: 4px; }
      #blogbody .posting .postbody a {
        text-decoration: none;
        color: #cc0000; }
        #blogbody .posting .postbody a:visited {
          color: #aa0000; }
        #blogbody .posting .postbody a:hover {
          color: #ff190d;
          text-shadow: #ff6666 0px 0px 1px; }
      #blogbody .posting .postbody p {
        line-height: 1.3em; }
      #blogbody .posting .postbody dl {
        padding-left: 16px; }
        #blogbody .posting .postbody dl dt {
          font-weight: bold;
          color: #444444;
          padding: 6px 92px 3px 0;
          padding-top: 9px; }
      #blogbody .posting .postbody ul {
        padding: 10px 0 10px 40px;
        list-style-type: disc; }
      #blogbody .posting .postbody li {
        padding-top: 3px; }
      #blogbody .posting .postbody pre, #blogbody .posting .postbody code {
        font-family: Bitstream Vera Sans Mono, monospace;
        font-size: 13px; }
      #blogbody .posting .postbody blockquote {
        border-left: 3pt solid #aaaaaa;
        padding-left: 1em;
        padding-right: 1em;
        margin-left: 1em;
        font: italic small Verdana, Times, sans-serif;
        color: #222222; }
        #blogbody .posting .postbody blockquote em {
          font-weight: bold; }
      #blogbody .posting .postbody img {
        max-width: 700px;
        margin: 0em 20px;
        -moz-box-shadow: 0px 0px 9px black;
        -webkit-box-shadow: 0px 0px 9px black;
        box-shadow: 0px 0px 4px black; }
      #blogbody .posting .postbody .amp {
        color: #666666;
        font-family: "Warnock Pro", "Goudy Old Style", "Palatino", "Book Antiqua", serif;
        font-style: italic; }
      #blogbody .posting .postbody .caps {
        font-size: 0.92em; }
      #blogbody .posting .postbody .highlight {
        border: 1px solid #cccccc;
        padding-left: 1em;
        margin-bottom: 10px;
        margin-top: 10px;
        background: none repeat scroll 0 0 #f0f0f0;
        overflow: auto; }
    #blogbody .posting .postfooter p {
      margin: 0 0 0 0;
      font-style: italic; }
    #blogbody .posting .postfooter a {
      color: #111111;
      font-family: Palatino, "times new roman", serif;
      font-weight: bold;
      font-size: 0.9em;
      text-decoration: none; }
      #blogbody .posting .postfooter a:hover {
        color: #111111;
        text-shadow: #aaaaaa 0px 0px 1px; }
  #blogbody .page {
    margin-top: -20px;
    font: bold medium Palatino, "Times New Roman", serif;
    color: black;
    text-decoration: none;
    border-bottom: 1px dotted black; }
    #blogbody .page:hover {
      text-shadow: #aaaaaa 0px 0px 1px; }

#blogfooter {
  margin-top: 10px;
  padding: 10px;
  text-align: center;
  font: medium Palatino, "Times New Roman", serif; }
  #blogfooter a {
    font-weight: bold;
    text-decoration: none;
    border-bottom: 1px dotted black;
    color: #cc0000; }
    #blogfooter a:hover {
      text-shadow: #ee6688 0px 0px 1px;
      color: #ff190d; }

/* article listing */
#arc {
  border-collapse: collapse; }
  #arc th, #arc td {
    padding-top: 0.4em;
    list-style: none;
    margin: 0; }
  #arc th {
    text-align: right;
    padding: 0.4em 1.75em 0 0;
    vertical-align: baseline; }

#disqus_thread {
  padding-top: 24px; }

object[type="application/x-shockwave-flash"] {
  -moz-box-shadow: 0px 0px 12px #777777;
  -webkit-box-shadow: 0px 0px 12px #777777;
  box-shadow: 0px 0px 12px #777777;
  margin-left: 15px;
  text-align: center; }

.shadow {
  -moz-box-shadow: 0px 0px 9px black;
  -webkit-box-shadow: 0px 0px 9px black;
  box-shadow: 0px 0px 4px black; }

.floatright {
  float: right; }

.floatleft {
  float: left; }
